# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 <Conor Drew> <the.ninja.circus@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk # pylint: disable=E0611

from nudge_lib.helpers import get_builder

import gettext
from gettext import gettext as _
gettext.textdomain('nudge')

import datetime

class NewReminderDialog(Gtk.Dialog):
    __gtype_name__ = "NewReminderDialog"

    def __new__(cls):
        """Special static method that's automatically called by Python when 
        constructing a new instance of this class.
        
        Returns a fully instantiated NewReminderDialog object.
        """
        builder = get_builder('NewReminderDialog')
        new_object = builder.get_object('new_reminder_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        """Called when we're finished initializing.

        finish_initalizing should be called after parsing the ui definition
        and creating a NewReminderDialog object with it in order to
        finish initializing the start of the new NewReminderDialog
        instance.
        """
        # Get a reference to the builder and set up the signals.
        self.builder = builder
        self.ui = builder.get_ui(self)
        
        date = datetime.datetime.now()
        
        self.minute = None
        self.remind_timeamount = None
        
        self.new_nudge_entry = self.builder.get_object("new_nudge_entry")
        self.new_nudge_entry.connect("changed", self.on_new_nudge_entry_changed)
        
        self.ui.btn_ok.set_sensitive(False)
        
        self.calendar = self.builder.get_object("calendar")
        self.calendar.select_month((date.month - 1), date.year)
        self.calendar.select_day(date.day)
        
        minute_store = Gtk.ListStore(str, int)
        displayminuteiter = minute_store.append([": 00", 0])
        minute_store.append([": 10", 10])
        minute_store.append([": 15", 15])
        minute_store.append([": 20", 20])
        minute_store.append([": 30", 30])
        minute_store.append([": 40", 40])
        minute_store.append([": 45", 45])
        minute_store.append([": 50", 50])
        self.hbox = self.builder.get_object("box5")
        self.minute_combo = Gtk.ComboBox.new_with_model(minute_store)
        self.minute_combo.connect("changed", self.on_minute_combo_changed)
        text_renderer = Gtk.CellRendererText()
        self.minute_combo.pack_start(text_renderer, True)
        self.minute_combo.add_attribute(text_renderer, "text", 0)
        self.minute_combo.set_active_iter(displayminuteiter)
        self.hbox.pack_start(self.minute_combo, True, True, 2)
        
        self.minute_combo.show()
        
        remind_store = Gtk.ListStore(str, int)
        displayreminditer = remind_store.append(["At this time", 0])
        remind_store.append(["15 minutes before", 15])
        remind_store.append(["30 minutes before", 30])
        remind_store.append(["45 minutes before", 45])
        remind_store.append(["An hour before", 1])
        self.hboxtwo = self.builder.get_object("box11")
        self.remind_combo = Gtk.ComboBox.new_with_model(remind_store)
        self.remind_combo.connect("changed", self.on_remind_combo_changed)
        self.remind_combo.pack_start(text_renderer, True)
        self.remind_combo.add_attribute(text_renderer, "text", 0)
        self.remind_combo.set_active_iter(displayreminditer)
        self.hboxtwo.pack_start(self.remind_combo, True, True, 0)
        
        self.remind_combo.show()
        
        self.am_pm_selector = Gtk.ComboBoxText()
        self.am_pm_selector.append_text("AM")
        self.am_pm_selector.append_text("PM")
        self.am_pm_selector.set_active(0)
        self.hbox.pack_start(self.am_pm_selector, True, True, 3)
        
        self.am_pm_selector.show()
        
    def on_minute_combo_changed(self, combo):
        tree_iter = combo.get_active_iter()
        if tree_iter != None:
            model = combo.get_model()
            self.minute = model[tree_iter][1]

    def on_remind_combo_changed(self, combo):
        tree_iter = combo.get_active_iter()
        if tree_iter != None:
            model = combo.get_model()
            self.remind_timeamount = model[tree_iter][1]
        
    def on_new_nudge_entry_changed(self, widget, data=None):
        self.ui.btn_ok.set_sensitive(True)

    def on_btn_ok_clicked(self, widget, data=None):
        """The user has elected to save the changes.

        Called before the dialog returns Gtk.ResponseType.OK from run().
        """
        pass

    def on_btn_cancel_clicked(self, widget, data=None):
        """The user has elected to cancel the changes.

        Called before the dialog returns Gtk.ResponseType.CANCEL for run()
        """
        pass

    @property
    def noon_status(self):
        return self.am_pm_selector.get_active_text()

    @property
    def title_text(self):
        return self.new_nudge_entry.get_text()

    @property
    def get_minute(self):
        return self.minute

    @property
    def hour_root_value(self):
        return self.ui.hour_adjustment.get_value()

    @property
    def new_nudge_date(self):
        (y, mo, d) = self.calendar.get_date()
        return (y, mo, d)

if __name__ == "__main__":
    dialog = NewReminderDialog()
    dialog.show()
    Gtk.main()
