# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2012 <Conor Drew> <the.ninja.circus@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os

import datetime, time

from gi.repository import AppIndicator3 # pylint: disable=E0611

import shelve

from gi.repository import Notify

import gettext
from gettext import gettext as _
gettext.textdomain('nudge')

from gi.repository import GLib # pylint: disable=E0611
from gi.repository import Gtk # pylint: disable=E0611
import logging
logger = logging.getLogger('nudge')

from nudge_lib.helpers import get_media_file

from nudge_lib import Window
from nudge.AboutNudgeDialog import AboutNudgeDialog
from nudge.PreferencesNudgeDialog import PreferencesNudgeDialog
from nudge.NewReminderDialog import NewReminderDialog

# See nudge_lib.Window.py for more details about how this class works
class NudgeWindow(Window):
    __gtype_name__ = "NudgeWindow"
    
    def finish_initializing(self, builder): # pylint: disable=E1002
        """Set up the main window"""
        super(NudgeWindow, self).finish_initializing(builder)

        self.AboutDialog = AboutNudgeDialog
        self.PreferencesDialog = PreferencesNudgeDialog

        # Code for other initialization actions should be added here.

        self.treeiter = 0

        self.menu_delete_iter = 0
        self.nudges_added = 0

        self.NudgeTitles = []
        self.NudgeEventTimes = []
        self.NudgeEverydays = []
        self.NudgeRemindTimes = []
        self.TodaysNudges = []
        self.current_reminder = []
        self.fast_nudge = [None, None, None, None]
        
        self.current_id = 0
        
        self.reminder_notification_id = 0

        load_db = shelve.open("Nudges", 'r')
        self.NudgeTitles = load_db["Titles"]
        self.NudgeEventTimes = load_db["EventTimes"]
        self.NudgeEverydays = load_db["EverydayStatuses"]
        self.NudgeRemindTimes = load_db["RemindTimes"]
        load_db.close()

        self.todaysdate = datetime.date.today()
        
        self.nudge_store = Gtk.ListStore(str, str, bool)

        self.indicator_menu = self.indicator.get_menu()
        self.current_menuitem = Gtk.MenuItem("No Upcoming Reminders")
        self.current_menuitem.set_sensitive(False)
        self.current_menuitem.connect("activate", self.on_current_menuitem_activate)
        self.current_menuitem.show()
        self.indicator_menu.prepend(self.current_menuitem)

        nudge_icon_uri = get_media_file("nudge.svg")
        self.nudge_icon_path = nudge_icon_uri.replace("file:///", '')

        Notify.init("nudge")

        self.toolbar = self.builder.get_object("toolbar")
        self.viewport = self.builder.get_object("viewport")
        self.delete_button = self.builder.get_object("delete_nudge_button")
        self.toolseparator = self.builder.get_object("toolbutton1")
        
        self.toolseparator.hide()
        self.delete_button.hide()
        
        self.connect("delete-event", self.delete_event)

        context = self.toolbar.get_style_context()
        context.add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
        
        self.menu_store = Gtk.ListStore(str, int)
        
        self.nudge_view = Gtk.TreeView(model=self.nudge_store)
        
        nudge_txtrender = Gtk.CellRendererText()
        due_txtrender = Gtk.CellRendererText()
        nudge_txtrender.set_property("ellipsize", "end")
        nudge_column = Gtk.TreeViewColumn("I Need To...", nudge_txtrender, text=0)
        due_column = Gtk.TreeViewColumn("By...", due_txtrender, text=1)
        self.nudge_view.append_column(nudge_column)
        nudge_column.set_min_width(143)
        due_column.set_min_width(232)
        due_column.set_expand(True)
        due_column.set_sort_column_id(1)
        self.nudge_view.append_column(due_column)
        self.nudge_view.set_rules_hint(True)
        
        self.viewport.add(self.nudge_view)
        
        self.nudge_view.show()
        
        def compare(model, row1, row2, user_data):
            sort_column, _ = model.get_sort_column_id()
            value1 = model.get_value(row1, sort_column)
            value2 = model.get_value(row2, sort_column)
            if model[row1][2] == True:
                value1i = datetime.datetime.strptime(str(value1), "%I:%M %p, Everyday")
            elif model[row1][2] == False:
                value1i = datetime.datetime.strptime(str(value1), "%I:%M %p; %A, %B %d, %Y")
            if model[row2][2] == True:
                value2i = datetime.datetime.strptime(str(value2), "%I:%M %p, Everyday")
            elif model[row2][2] == False:
                value2i = datetime.datetime.strptime(str(value2), "%I:%M %p; %A, %B %d, %Y")
            if value1i < value2i:
                return -1
            elif value1i == value2i:
                return 0
            else:
                return 1
        
        self.nudge_store.set_sort_func(1, compare, None)
        
        select = self.nudge_view.get_selection()
        select.set_mode(Gtk.SelectionMode.BROWSE)
        select.connect("changed", self.on_tree_selection_changed)       
   
        data_size = len(self.NudgeRemindTimes)
        
        if data_size != 0:
            self.init_loaded_nudges()
            self.sort_for_new_current()
    
    def init_loaded_nudges(self):
        for i in range(len(self.NudgeRemindTimes)):
            if self.NudgeEverydays[i] == True:
                self.NudgeRemindTimes[i] = datetime.datetime(self.todaysdate.year, self.todaysdate.month, self.todaysdate.day, self.NudgeRemindTimes[i].hour, self.NudgeRemindTimes[i].minute)
            if self.NudgeRemindTimes[i] > datetime.datetime.now() or self.NudgeEverydays[i] == True:
                self.treeiter = self.nudge_store.append([self.NudgeTitles[i], self.NudgeEventTimes[i], self.NudgeEverydays[i]])
                self.nudges_added += 1
                if self.NudgeRemindTimes[i].day == self.todaysdate.day and self.NudgeRemindTimes[i].month == self.todaysdate.month:
                    self.TodaysNudges.append([self.NudgeTitles[i], self.NudgeEventTimes[i], self.NudgeEverydays[i], self.NudgeRemindTimes[i]])
                    self.menu_delete_iter = 1
            else:
                self.delete(self.NudgeTitles[i])
                self.save()
                return

    def sort_for_new_current(self):
        time_differences = []
        for i in self.TodaysNudges:
            time_differences.append(i[3] - datetime.datetime.now())
        if len(time_differences) > 0:
            sorted_diff = sorted(time_differences)
            min_diff = sorted_diff[0]
            for i, s in enumerate(time_differences):
                if s == min_diff:
                    self.current_id = i
            self.current_reminder = self.TodaysNudges[self.current_id]
            self.current_menuitem.set_label(self.current_reminder[0])
            self.current_menuitem.show()
   
    def show_reminder_notifications(self, nudge):
        if self.nudges_added == 0:
            return
        title = nudge[0]
        time = nudge[1]
        reminder = Notify.Notification.new(title, time, self.nudge_icon_path)
        reminder.show()
        #self.indicator.set_status(AppIndicator3.IndicatorStatus.ATTENTION)
        self.delete(self.current_reminder[0])
        
    def add_nudge(self, fastnudge):
        self.NudgeTitles.append(fastnudge[0])
        self.NudgeEventTimes.append(fastnudge[1])
        self.NudgeEverydays.append(fastnudge[2])
        self.NudgeRemindTimes.append(fastnudge[3])

    def save(self):
        save_db = shelve.open("Nudges", 'w')
        save_db["Titles"] = self.NudgeTitles
        save_db["EventTimes"] = self.NudgeEventTimes
        save_db["EverydayStatuses"] = self.NudgeEverydays
        save_db["RemindTimes"] = self.NudgeRemindTimes
        save_db.close()

    ####JUST SHOW AND DELETE CURRENT REMINDER self.current_menuitem(self.current_reminder[0])

    def delete(self, nudge_title):
        self.nudges_added -= 1
        delete_today_list = self.TodaysNudges
        for i in range(len(delete_today_list)):
            if delete_today_list[i][0] == nudge_title:
                del delete_today_list[i]
        self.TodaysNudges = delete_today_list
        for row in self.nudge_store:
            if row[0] == nudge_title:
                self.nudge_store.remove(row.iter)
        for i, s in enumerate(self.NudgeTitles):
            if s == nudge_title:
                del_index = i
        del self.NudgeTitles[del_index]
        del self.NudgeEventTimes[del_index]
        del self.NudgeEverydays[del_index]
        del self.NudgeRemindTimes[del_index]
        self.save()
        if self.nudges_added == 0:
            self.current_menuitem.set_label("No Upcoming Reminders")
            self.current_menuitem.show()
        else:
            self.sort_for_new_current()

    #def add_nudge_entry(self, title, ident):
    #    self.menutreeiter = self.menu_store.append([title, ident])
    #    if self.menu_delete_iter == 0:
    #        add_menu_list = self.indicator_menu.get_children()
    #        self.indicator_menu.remove(add_menu_list[0])
    #        self.menu_delete_iter = 1
    #    new_nudge_menuentry = Gtk.MenuItem(title)
    #    new_nudge_menuentry.set_sensitive(False)
    #    new_nudge_menuentry.show()
    #    self.indicator_menu.prepend(new_nudge_menuentry)
    #    self.indicator.set_menu(self.indicator_menu)
        
    def on_new_nudge_button_clicked(self, widget, data=None):
        ##Start New Reminder Dialog
        new_reminder = NewReminderDialog()
        result = new_reminder.run()
        ####Get everything from the dialog
        nudge_title = new_reminder.title_text
        hours_abstract = new_reminder.hour_root_value
        minutes_abstract = new_reminder.get_minute
        am_pm_status = new_reminder.noon_status
        nudge_date = new_reminder.new_nudge_date
        everyday_toggle = new_reminder.builder.get_object("everyday_toggle")
        everytog = everyday_toggle.get_active()
        remind_time_abstract = new_reminder.remind_timeamount
        new_reminder.destroy()
        if result != Gtk.ResponseType.OK:
            return
        ####Use the data from the dialog
        self.fast_nudge[0] = nudge_title
        if am_pm_status == "AM" and int(hours_abstract) == 12:
            hours_abstract = 0
        if am_pm_status == "PM" and int(hours_abstract) != 12:
            hours_abstract += 12
        time_abstract = datetime.time(int(hours_abstract), minutes_abstract)
        time_string = time_abstract.strftime("%I:%M %p")
        (year, month, day) = nudge_date
        month = month + 1
        preformat_date = datetime.date(year, month, day)
        format_date = preformat_date.strftime("%A, %B %d, %Y")
        self.fast_nudge[2] = False
        if everytog == True:
            self.fast_nudge[2] = True
            self.fast_nudge[1] = time_string + ", Everyday"
            precheck_time = datetime.datetime.strptime(str(self.fast_nudge[1]), "%I:%M %p, Everyday")
            check_time = datetime.datetime(self.todaysdate.year, self.todaysdate.month, self.todaysdate.day, precheck_time.hour, precheck_time.minute)
        elif everytog == False:
            self.fast_nudge[1] = time_string + '; ' + format_date
            check_time = datetime.datetime.strptime(str(self.fast_nudge[1]), "%I:%M %p; %A, %B %d, %Y")
        if remind_time_abstract == 1:
            remind_offset = datetime.time(remind_time_abstract)
        else:
            remind_offset = datetime.time(0, remind_time_abstract)
        ###############Determine nudge_remind_time / fast_nudge[3] here
        self.fast_nudge[3] = check_time - datetime.timedelta(hours = remind_offset.hour, minutes = remind_offset.minute)
        self.treeiter = self.nudge_store.append([self.fast_nudge[0], self.fast_nudge[1], self.fast_nudge[2]])
        self.add_nudge(self.fast_nudge)
        self.nudges_added += 1
        self.save()
        if self.fast_nudge[3].day == self.todaysdate.day and self.fast_nudge[3].month == self.todaysdate.month:
            self.TodaysNudges.append(self.fast_nudge)
            self.sort_for_new_current()
        return
    
    #self.indicator.foo -- access the indicator from this file
    #self.nudge_list[self.treeiter][COLUMN_NUMBER] - for future reference

    def delete_event(self, window, event):
        self.hide_on_delete()
        self.hide_sep = Gtk.SeparatorMenuItem(self.indicator_menu)
        self.hide_sep.show()
        self.indicator_menu.prepend(self.hide_sep)
        self.show_nudge = Gtk.MenuItem("Show Nudge")
        self.show_nudge.connect("activate", self.show_all)
        self.show_nudge.show()
        self.indicator_menu.prepend(self.show_nudge)
        self.indicator.set_menu(self.indicator_menu)
        return True
        
    def show_all(self, window):
        self.show()
        show_remove_list = self.indicator_menu.get_children()
        self.indicator_menu.remove(show_remove_list[0])
        self.indicator_menu.remove(show_remove_list[1])
        self.indicator.set_menu(self.indicator_menu)
        return
        
    def on_tree_selection_changed(self, selection):
        model, triter = selection.get_selected()
        if triter != None:
            self.toolseparator.show()
            self.delete_button.show()
        else:
            self.toolseparator.hide()
            self.delete_button.hide()

    ####menu_item.get_label()
            
    def on_delete_nudge_button_clicked(self, widget, data=None):
        selection = self.nudge_view.get_selection()
        result = selection.get_selected()
        if result: #result could be None
            model, listiter = result
            remove_title = model[listiter][0]
            model.remove(listiter)
            self.delete(remove_title)
        #else:
        #    print "model[listiter][3]: ", model[listiter][3]
        #    self.delete((model[listiter][3])-1)
        
    def on_current_menuitem_activate(self, menuitem):
        #delete_find_list = []
        #for i in self.TodaysNudges:
        #    delete_find_list.append(i[0])
        #delete_index = delete_find_list.index(self.current_menuitem.get_label())
        #self.delete(delete_index)
        self.indicator.set_status(AppIndicator3.IndicatorStatus.ACTIVE)
        
    def check_for_reminder(self):
        for i in self.TodaysNudges:
            if i[3].hour == datetime.datetime.now().hour and i[3].minute == datetime.datetime.now().minute:
                self.show_reminder_notifications(i)
